@@ -21,7 +21,7 @@
 
 #define CROSS_SZ 5
 
-#define LOGO_NAME	"linuxlogo.bmp"
+#define LOGO_NAME	"linuxlogo.bitmap"
 #define LOGO_WIDTH	201
 #define LOGO_HEIGHT	85
 
@@ -464,7 +464,7 @@ void GetLogo(void) {
     fd = fopen(LOGO_NAME, "rb");
     if (!fd) {
       read_err:
-	perror("Problems reading linuxlogo.bmp");
+	perror("Problems reading linuxlogo.bitmap");
 	exit(2);
     }
     if (1 != fread(blitimage, LOGO_WIDTH * LOGO_HEIGHT, 1, fd))
