@@ -12,6 +12,18 @@
 #include <linux/pci.h>
 #include <linux/devfs_fs_kernel.h>
 
+#ifndef _LINUX_DEVFS_FS_KERNEL_H
+static inline int devfs_register_chrdev (unsigned int major, const char *name,
+                                         struct file_operations *fops)
+{
+    return register_chrdev (major, name, fops);
+}
+static inline int devfs_unregister_chrdev (unsigned int major,const char *name)
+{
+    return unregister_chrdev (major, name);
+}
+#endif
+
 #include <asm/uaccess.h>
 #include <asm/system.h>   /* cli(), *_flags */
 #include <asm/segment.h>  /* memcpy and such */
@@ -273,12 +285,14 @@ int init_module(void)
 {
     int result, i;
     struct pci_dev *dev=NULL;
-
+#ifdef CONFIG_DEVFS_FS
+    devfs_handle_t slave_handle;
+#endif
     /*
      * Register your major, and accept a dynamic number
      */
      
-    printk(KERN_INFO "svgalib_helper: Initializing, version 1.9.13\n");
+    printk(KERN_INFO "svgalib_helper: Initializing, version 1.9.14\n");
     result = devfs_register_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper", &svgalib_helper_fops);
     if (result < 0) {
         printk(KERN_WARNING "svgalib_helper: can't get major %d\n",SVGALIB_HELPER_MAJOR);
@@ -297,7 +311,8 @@ int init_module(void)
     devfs_register_series( devfs_handle,
                "%u", 8, DEVFS_FL_DEFAULT, SVGALIB_HELPER_MAJOR, 0,
                        S_IFCHR |  S_IRUGO | S_IRWXU, &svgalib_helper_fops, NULL ) ;
-    devfs_mk_symlink( NULL, "svga", 0, "svga_helper/0", NULL, NULL );
+    devfs_mk_symlink( NULL, "svga", 0, "svga_helper/0", &slave_handle, NULL );
+    devfs_auto_unregister( devfs_handle, slave_handle );
 #endif /* devfsd support */
 
     if(pci_present()) {
@@ -360,3 +375,8 @@ void cleanup_module(void)
 #endif
     devfs_unregister_chrdev(SVGALIB_HELPER_MAJOR, "svgalib_helper");
 }
+
+#ifdef MODULE_LICENSE
+MODULE_LICENSE("GPL and additional rights");
+#endif
+
