@@ -82,7 +82,6 @@ extern DriverSpecs __svgalib_mach64_driverspecs;
 extern DriverSpecs __svgalib_chips_driverspecs;
 extern DriverSpecs __svgalib_apm_driverspecs;
 extern DriverSpecs __svgalib_nv3_driverspecs;
-extern DriverSpecs __svgalib_g400_driverspecs;
 extern DriverSpecs __svgalib_et6000_driverspecs;
 extern DriverSpecs __svgalib_vesa_driverspecs;
 extern DriverSpecs __svgalib_mx_driverspecs;
@@ -122,7 +121,7 @@ enum {
 };
 
 enum { CURSOR_INIT, CURSOR_HIDE, CURSOR_SHOW, CURSOR_POSITION, 
-       CURSOR_SELECT, CURSOR_IMAGE 
+       CURSOR_SELECT, CURSOR_IMAGE, CURSOR_SAVE
 };
 
 typedef struct {
