@@ -296,7 +296,7 @@ static int init(int force, int par1, int par2)
     }
 
     if (__svgalib_driver_report) {
-	printf("Using SK driver, %iKB. ",memory);
+	fprintf(stderr,"Using SK driver, %iKB.\n",memory);
     };
 
     cardspecs = malloc(sizeof(CardSpecs));
