@@ -64,8 +64,12 @@
 	sigaction(sig, &sa, NULL); \
 }
 
-/* variables used to shift between monchrome and color emulation */
+#undef SAVEREGS
+#ifdef SAVEREGS
+static unsigned char graph_regs[16384];
+#endif
 
+/* variables used to shift between monchrome and color emulation */
 int __svgalib_CRT_I;			/* current CRT index register address */
 int __svgalib_CRT_D;			/* current CRT data register address */
 int __svgalib_IS1_R;			/* current input status register address */
@@ -414,7 +418,8 @@ int mouse_open = 0;
 static int mouse_mode = 0;
 static int mouse_type = -1;
 static int mouse_modem_ctl = 0;
-static char *mouse_device = "/dev/mouse";
+char *__svgalib_mouse_device = "/dev/mouse";
+int __svgalib_mouse_flag;
 static char *mem_device = "/dev/svga";
 static int __svgalib_oktowrite = 1;
 static int modeinfo_mask = ~0;
@@ -748,7 +753,7 @@ static void open_mem(void)
 
     if (__svgalib_mem_fd < 0)
 	if ((__svgalib_mem_fd = open(mem_device, O_RDWR)) < 0) {
-	    printf("svgalib: Cannot open %s\n",mem_device);
+	    fprintf(stderr,"svgalib: Cannot open %s\n",mem_device);
 	    exit(1);
 	}
 }
@@ -976,7 +981,7 @@ static void signal_handler(int v)
     idle_accel();
     
     restoretextmode();
-    printf("svgalib: Signal %d: %s received%s.\n", v, strsignal(v),
+    fprintf(stderr,"svgalib: Signal %d: %s received%s.\n", v, strsignal(v),
 	   (v == SIGINT) ? " (ctrl-c pressed)" : "");
 
     for (i = 0; i < sizeof(sig2catch); i++)
@@ -986,7 +991,7 @@ static void signal_handler(int v)
 	    break;
 	}
     if (i >= sizeof(sig2catch)) {
-	printf("svgalib: Aieeee! Illegal call to signal_handler, raising segfault.\n");
+	fprintf(stderr,"svgalib: Aieeee! Illegal call to signal_handler, raising segfault.\n");
 	raise(SIGSEGV);
     }
 }
@@ -1197,12 +1202,12 @@ void vga_setchipset(int c)
 {
     CHIPSET = c;
 #ifdef DEBUG
-    printf("Setting chipset\n");
+    fprintf(stderr,"Setting chipset\n");
 #endif
     if (c == UNDEFINED)
 	return;
     if (__svgalib_driverspecslist[c] == NULL) {
-	printf("svgalib: Invalid chipset. The driver may not be compiled in.\n");
+	fprintf(stderr,"svgalib: Invalid chipset. The driver may not be compiled in.\n");
 	CHIPSET = UNDEFINED;
 	return;
     }
@@ -1217,12 +1222,12 @@ void vga_setchipsetandfeatures(int c, int par1, int par2)
 {
     CHIPSET = c;
 #ifdef DEBUG
-    printf("Forcing chipset and features\n");
+    fprintf(stderr,"Forcing chipset and features\n");
 #endif
     __svgalib_get_perm();
     __svgalib_driverspecslist[c]->init(1, par1, par2);
 #ifdef DEBUG
-    printf("Finished forcing chipset and features\n");
+    fprintf(stderr,"Finished forcing chipset and features\n");
 #endif
     __svgalib_setpage = __svgalib_driverspecs->__svgalib_setpage;
     __svgalib_setrdpage = __svgalib_driverspecs->__svgalib_setrdpage;
@@ -1289,7 +1294,7 @@ static void __svgalib_releasevt_signal(int n)
 	return;
     }
 #ifdef DEBUG
-    printf("Release request.\n");
+    fprintf(stderr,"Release request.\n");
 #endif
 release_acquire=!release_acquire;
     forbidvtacquire = 1;
@@ -1311,27 +1316,27 @@ release_acquire=!release_acquire;
         };
     ioctl(__svgalib_tty_fd, VT_RELDISP, 1);
 #ifdef DEBUG
-    printf("Finished release.\n");
+    fprintf(stderr,"Finished release.\n");
 #endif
     forbidvtacquire = 0;
 
     /* Suspend program until switched to again. */
 #ifdef DEBUG
-    printf("Suspended.\n");
+    fprintf(stderr,"Suspended.\n");
 #endif
 
     __svgalib_oktowrite = 0;
     if (!__svgalib_runinbackground)
 	__svgalib_waitvtactive();
 #ifdef DEBUG
-    printf("Waked.\n");
+    fprintf(stderr,"Waked.\n");
 #endif
 }
 
 static void __svgalib_acquirevt_signal(int n)
 {
 #ifdef DEBUG
-    printf("Acquisition request.\n");
+    fprintf(stderr,"Acquisition request.\n");
 #endif
 release_acquire=!release_acquire;
     forbidvtrelease = 1;
@@ -1342,7 +1347,7 @@ release_acquire=!release_acquire;
     __svgalib_flipback();
     ioctl(__svgalib_tty_fd, VT_RELDISP, VT_ACKACQ);
 #ifdef DEBUG
-    printf("Finished acquisition.\n");
+    fprintf(stderr,"Finished acquisition.\n");
 #endif
     forbidvtrelease = 0;
     if (__svgalib_come_from_background)
@@ -1512,7 +1517,7 @@ static void initialize(void)
 
 #if 0 /* What is this for? */
     if ((long) GM < 0) {
-	printf("svgalib: mmap error rrr\n");
+	fprintf(stderr,"svgalib: mmap error rrr\n");
 	exit(1);
     }
 #endif
@@ -1520,19 +1525,20 @@ static void initialize(void)
     vga_screenoff();
 
     /* Sanity check: (from painful experience) */
-
     i = __svgalib_saveregs(text_regs);
     if (i > MAX_REGS) {
-	puts("svgalib: FATAL internal error:");
-	printf("Set MAX_REGS at least to %d in src/driver.h and recompile everything.\n",
+	fprintf(stderr,"svgalib: FATAL internal error:\n");
+	fprintf(stderr,"Set MAX_REGS at least to %d in src/driver.h and recompile everything.\n",
 	       i);
 	exit(1);
     }
+
     /* This appears to fix the Trident 8900 rebooting problem. */
     if (__svgalib_chipset == TVGA8900) {
 	text_regs[EXT + 11] = __svgalib_inseq(0x0c);
 	text_regs[EXT + 12] = __svgalib_incrtc(0x1f);
     }
+
     /* save text mode palette */
     savepalette(text_red, text_green, text_blue);
 
@@ -1613,7 +1619,7 @@ void vga_safety_fork(void (*shutdown_routine) (void))
     int child_status, oldkbmode;
 
     if (initialized) {
-	printf("svgalib: warning: vga_safety_fork() called when already initialized\n");
+	fprintf(stderr,"svgalib: warning: vga_safety_fork() called when already initialized\n");
 	goto no_fork;
     }
     initialize();
@@ -1628,7 +1634,7 @@ void vga_safety_fork(void (*shutdown_routine) (void))
     childpid = fork();
     if (childpid < 0) {
       no_fork:
-	printf("svgalib: warning: can't fork to enhance reliability; proceeding anyway");
+	fprintf(stderr,"svgalib: warning: can't fork to enhance reliability; proceeding anyway");
 	return;
     }
     if (childpid) {
@@ -1646,20 +1652,20 @@ void vga_safety_fork(void (*shutdown_routine) (void))
 		exit(WEXITSTATUS(child_status));
 
 	    if (WCOREDUMP(child_status))
-		puts("svgalib:vga_safety_fork: Core dumped!");
+		fprintf(stderr,"svgalib:vga_safety_fork: Core dumped!\n");
 
 	    if (WIFSIGNALED(child_status)) {
-		printf("svgalib:vga_safety_fork: Killed by signal %d, %s.\n",
+		fprintf(stderr,"svgalib:vga_safety_fork: Killed by signal %d, %s.\n",
 		       WTERMSIG(child_status),
 		       strsignal(WTERMSIG(child_status)));
 		exit(1);
 	    }
 	    if (WIFSTOPPED(child_status)) {
-		printf("svgalib:vga_safety_fork: Stopped by signal %d, %s.\n",
+		fprintf(stderr,"svgalib:vga_safety_fork: Stopped by signal %d, %s.\n",
 		       WSTOPSIG(child_status),
 		       strsignal(WSTOPSIG(child_status)));
-		puts("\aWARNING! Continue stopped svgalib application at own risk. You are better\n"
-		     "off killing it NOW!");
+		fprintf(stderr,"\aWARNING! Continue stopped svgalib application at own risk. You are better\n"
+		     "off killing it NOW!\n");
 		continue;
 	    }
 	}
@@ -2084,7 +2090,7 @@ int vga_screenoff(void)
 	tmp = __svgalib_driverspecs->emul->screenoff();
     } else {
 	/* turn off screen for faster VGA memory acces */
-	if (CHIPSET != EGA) {
+	if ((CHIPSET != EGA) && !__svgalib_novga) {
             __svgalib_outseq(0x01,__svgalib_inseq(0x01) | 0x20);
 	}
 	/* Disable video output */
@@ -2107,7 +2113,7 @@ int vga_screenon(void)
 	tmp = __svgalib_driverspecs->emul->screenon();
     } else {
 	/* turn screen back on */
-	if (CHIPSET != EGA) {
+	if ((CHIPSET != EGA) && !__svgalib_novga) {
             __svgalib_outseq(0x01,__svgalib_inseq(0x01) & 0xdf);
 	}
 /* #ifdef DISABLE_VIDEO_OUTPUT */
@@ -2202,6 +2208,7 @@ static int saved_page;
 static int saved_logicalwidth;
 static int saved_displaystart;
 static int saved_modeX;
+static char saved_crtc[CRT_C];
 
 static void savestate(void)
 {
@@ -2216,12 +2223,16 @@ static void savestate(void)
     saved_displaystart = currentdisplaystart;
     saved_modeX = MODEX;
 
+    if(STDVGAMODE(CM)) {
+        vga_getcrtcregs(saved_crtc); /* save for tweaked modes */
+    }
+
     if (CM == G320x200x256 && VMEM <= 65536) {
 	/* 320x200x256 is a special case; only 64K is addressable */
 	/* (unless more has been claimed, in which case we assume */
 	/* SVGA bank-switching) */
 	if ((graph_buf = malloc(GRAPH_SIZE)) == NULL) {
-	    printf("Cannot allocate memory for VGA state\n");
+	    fprintf(stderr,"Cannot allocate memory for VGA state\n");
 	    vga_setmode(TEXT);
 	    exit(1);
 	}
@@ -2230,7 +2241,7 @@ static void savestate(void)
 	/* for planar VGA modes, save the full 256K */
 	__svgalib_vga_driverspecs.setmode(GPLANE16, prv_mode);
 	if ((graph_buf = malloc(4 * GRAPH_SIZE)) == NULL) {
-	    printf("Cannot allocate memory for VGA state\n");
+	    fprintf(stderr,"Cannot allocate memory for VGA state\n");
 	    vga_setmode(TEXT);
 	    exit(1);
 	}
@@ -2245,7 +2256,7 @@ static void savestate(void)
 
 	size = VMEM;
 	if ((graph_buf = malloc(4 * size)) == NULL) {
-	    printf("Cannot allocate memory for VGA state\n");
+	    fprintf(stderr,"Cannot allocate memory for VGA state\n");
 	    vga_setmode(TEXT);
 	    exit(1);
 	}
@@ -2267,7 +2278,7 @@ static void savestate(void)
 
 	size = VMEM;
 	if ((graph_buf = malloc(size)) == NULL) {
-	    printf("Cannot allocate memory for SVGA state.\n");
+	    fprintf(stderr,"Cannot allocate memory for SVGA state.\n");
 	    vga_setmode(TEXT);
 	    exit(1);
 	}
@@ -2294,6 +2305,10 @@ static void restorestate(void)
     if (saved_modeX)
 	vga_setmodeX();
 
+    if(STDVGAMODE(CM)) {
+        vga_setcrtcregs(saved_crtc); /* restore tweaked modes */
+    }
+
     restorepalette(graph_red, graph_green, graph_blue);
 
     if (CM == G320x200x256 && VMEM <= 65536) {
@@ -2353,7 +2368,7 @@ static void restorestate(void)
 	size = VMEM;
 
 #ifdef DEBUG
-	printf("Restoring %dK of video memory.\n", (size + 2) / 1024);
+	fprintf(stderr,"Restoring %dK of video memory.\n", (size + 2) / 1024);
 #endif
 	page = 0;
 	while (size >= 65536) {
@@ -2406,7 +2421,7 @@ void __svgalib_flipaway(void)
 	/* wait for any blitter operation to finish */
 	idle_accel();
 	/* Save state and go to textmode. */
-#if 0
+#ifdef SAVEREGS
         chipset_saveregs(graph_regs);
         __svgalib_saveregs(graph_regs);
 #endif
@@ -2432,11 +2447,12 @@ static void __svgalib_flipback(void)
   	  flip = 1;
 	  vga_setmode(flip_mode);
 	  flip = 0;
-#if 0
+#ifdef SAVEREGS
           chipset_setregs(graph_regs,flip_mode);
           __svgalib_setregs(graph_regs);
 #endif
 	  restorestate();
+          if(__svgalib_cursor_status>=0) __svgalib_cursor_restore();
         }  
     }
 }
@@ -2632,11 +2648,11 @@ static void parse_string(char *str, char **commands, char *(*func) (int ind, int
     ptr = __svgalib_token(nptr);
     while (ptr) {
 #ifdef DEBUG_CONF
-	printf("Parsing: %s\n", ptr);
+	fprintf(stderr,"Parsing: %s\n", ptr);
 #endif
 	for (curr = commands, index = 0; *curr; curr++, index++) {
 #ifdef DEBUG_CONF
-	    printf("Checking: %s\n", *curr);
+	    fprintf(stderr,"Checking: %s\n", *curr);
 #endif
 	    if (**curr == '!') {
 		if (!strcmp(*curr + 1, ptr)) {
@@ -2666,7 +2682,7 @@ static void process_config_file(FILE *file, int mode, char **commands,
     
     fstat(fileno(file), &st);	/* Some error analysis may be fine here.. */
     if ( (buf = alloca(st.st_size + 1)) == 0) {	/* + a final \0 */
-        puts("svgalib: out of mem while parsing config file !");
+        fprintf(stderr,"svgalib: out of mem while parsing config file !\n");
         return;
     }
     fread(buf, 1, st.st_size, file);
@@ -2702,7 +2718,7 @@ void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode, c
     }
     if ( (file = fopen(configfilename, "r")) != 0) {
 #ifdef DEBUG_CONF
-  printf("Processing config file \'%s\'\n", configfilename);
+  fprintf(stderr,"Processing config file \'%s\'\n", configfilename);
 #endif
       process_config_file(file, 1, commands, func);
       fclose(file);
@@ -2716,13 +2732,13 @@ void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode, c
 
       filename = alloca(strlen(ptr) + 20);
       if (!filename) {
-	puts("svgalib: out of mem while parsing SVGALIB_CONFIG_FILE !");
+	fprintf(stderr,"svgalib: out of mem while parsing SVGALIB_CONFIG_FILE !\n");
       } else {
 	strcpy(filename, ptr);
 	strcat(filename, "/.svgalibrc");
 	if ( (file = fopen(filename, "r")) != 0) {
 #ifdef DEBUG_CONF
-	  printf("Processing config file \'%s\'\n", filename);
+	  fprintf(stderr,"Processing config file \'%s\'\n", filename);
 #endif
 	  process_config_file(file, allowoverride, commands, func);
 	  fclose(file);
@@ -2733,7 +2749,7 @@ void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode, c
     if ( (ptr = getenv("SVGALIB_CONFIG_FILE")) != 0) {
       if ( (file = fopen(ptr, "r")) != 0) {
 #ifdef DEBUG_CONF
-  printf("Processing config file \'%s\'\n", ptr);
+  fprintf(stderr,"Processing config file \'%s\'\n", ptr);
 #endif
 	process_config_file(file, allowoverride, commands, func);
 	fclose(file);
@@ -2745,11 +2761,11 @@ void __svgalib_read_options(char **commands, char *(*func) (int ind, int mode, c
     if ( (ptr = getenv("SVGALIB_CONFIG")) != 0  &&  (i = strlen(ptr)) != 0) {
       buf = alloca(i + 1);
       if (!buf) {
-	puts("svgalib: out of mem while parsing SVGALIB_CONFIG !");
+	fprintf(stderr,"svgalib: out of mem while parsing SVGALIB_CONFIG !\n");
       } else {
 	strcpy(buf, ptr);		/* Copy for safety and strtok!! */
 #ifdef DEBUG_CONF
-	puts("Parsing env variable \'SVGALIB_CONFIG\'");
+	fprintf(stderr,"Parsing env variable \'SVGALIB_CONFIG\'\n");
 #endif
 	parse_string(buf, commands, func, allowoverride);
       }
@@ -2831,17 +2847,17 @@ static char *process_option(int command, int mode, char **nptr)
     float f;
 
 #ifdef DEBUG_CONF
-    printf("command %d detected.\n", command);
+    fprintf(stderr,"command %d detected.\n", command);
 #endif
     switch (command) {
     case 5:
 #ifdef DEBUG_CONF
-	puts("Allow override");
+	fprintf(stderr,"Allow override\n");
 #endif
 	if (mode)
 	    allowoverride = 1;
 	else
-	    puts("Overrideenable denied. (Gee.. Do you think I'm that silly?)");
+	    fprintf(stderr,"Overrideenable denied. (Gee.. Do you think I'm that silly?)\n");
 	break;
     case 0:			/* mouse */
     case 2:			/* m */
@@ -2864,16 +2880,16 @@ static char *process_option(int command, int mode, char **nptr)
 		}
 	    }
 	  inv_mouse:
-	    printf("svgalib: Illegal mouse setting: {mouse|m} %s\n"
+	    fprintf(stderr,"svgalib: Illegal mouse setting: {mouse|m} %s\n"
 		   "Correct usage: {mouse|m} mousetype\n"
 		   "where mousetype is one of 0, 1, 2, 3, 4, 5, 6, 7, 9,\n",
 		   (ptr != NULL) ? ptr : "");
 	    for (tabptr = conf_mousenames, i = 0; *tabptr; tabptr++, i++) {
 		if (i == MOUSE_NONE)
 		    continue;
-		printf("%s, ", *tabptr);
+		fprintf(stderr,"%s, ", *tabptr);
 	    }
-	    puts("or none.");
+	    fprintf(stderr,"or none.\n");
 	    return ptr;		/* Allow a second parse of str */
 	}
 	break;
@@ -2896,7 +2912,7 @@ static char *process_option(int command, int mode, char **nptr)
 		goto mon_deny;
 	    __svgalib_horizsync.max = f * 1000.0f;
 	} else {
-	    printf("svgalib: Illegal monitor setting: {monitor|M} %s\n"
+	    fprintf(stderr,"svgalib: Illegal monitor setting: {monitor|M} %s\n"
 		   "Correct usage: {monitor|M} monitortype\n"
 		   "where monitortype is one of 0, 1, 2, 3, 4, 5, 6, or\n"
 		   "maximal horz. scan frequency in khz.\n"
@@ -2908,14 +2924,14 @@ static char *process_option(int command, int mode, char **nptr)
     case 4:			/* chipset */
 	ptr = __svgalib_token(nptr);
 	if (ptr == NULL) {
-	    puts("svgalib: Illegal chipset setting: no chipset given");
+	    fprintf(stderr,"svgalib: Illegal chipset setting: no chipset given\n");
 	    goto chip_us;
 	}
 	/*First param is chipset */
 	for (i = 0, tabptr = driver_names; *tabptr; tabptr++, i++) {
 	    if (!strcasecmp(ptr, *tabptr)) {
 		if (!__svgalib_driverspecslist[i]) {
-		    printf("svgalib: Illegal chipset setting: Driver for %s is NOT compiled in.\n",
+		    fprintf(stderr,"svgalib: Illegal chipset setting: Driver for %s is NOT compiled in.\n",
 			ptr);
 		    continue; /* The for above will loop a few more times and fail */
 		}
@@ -2928,35 +2944,35 @@ static char *process_option(int command, int mode, char **nptr)
 			    vga_setchipsetandfeatures(i, j, atoi(ptr));
 			else {
 			  chipdeny:
-			    puts("chipset override from environment denied.");
+			    fprintf(stderr,"chipset override from environment denied.\n");
 			}
 			return __svgalib_token(nptr);
 		    } else {
-			puts("svgalib: Illegal chipset setting: memory is not a number");
+			fprintf(stderr,"svgalib: Illegal chipset setting: memory is not a number\n");
 			goto chip_us;
 		    }
 		}
 		if (mode)
 		    vga_setchipset(i);
 		else
-		    puts("chipset override from environment denied.");
+		    fprintf(stderr,"chipset override from environment denied.\n");
 		return ptr;
 	    }
 	}
-	printf("svgalib: Illegal chipset setting: chipset %s\n", ptr);
+	fprintf(stderr,"svgalib: Illegal chipset setting: chipset %s\n", ptr);
       chip_us:
-	puts("Correct usage: chipset driver [par1 par2]\n"
-	     "where driver is one of:");
+	fprintf(stderr,"Correct usage: chipset driver [par1 par2]\n"
+	     "where driver is one of:\n");
 	ptb = "%s";
 	for (i = 0, tabptr = driver_names; *tabptr; tabptr++, i++) {
 	    if (__svgalib_driverspecslist[i] != NULL) {
-		printf(ptb, *tabptr);
+		fprintf(stderr,ptb, *tabptr);
 		ptb = ", %s";
 	    }
 	}
-	puts("\npar1 and par2 are river dependant integers.\n"
+	fprintf(stderr,"\npar1 and par2 are river dependant integers.\n"
 	     "Example: Chipset VGA    or\n"
-	     "Chipset VGA 0 512");
+	     "Chipset VGA 0 512\n");
 	return ptr;
     case 6:			/* oldstyle config: m0-m4 */
     case 7:
@@ -2978,7 +2994,7 @@ static char *process_option(int command, int mode, char **nptr)
       monnum:
 	if (!mode) {
 	  mon_deny:
-	    puts("Monitor setting from environment denied.");
+	    fprintf(stderr,"Monitor setting from environment denied.\n");
 	    break;
 	} else {
 	    __svgalib_horizsync.max = __svgalib_maxhsync[command - 12];
@@ -3007,14 +3023,14 @@ static char *process_option(int command, int mode, char **nptr)
     case 31:			/* c0-c1 color-text selection */
 	if (!mode) {
 	  coltexdeny:
-	    puts("Color/mono text selection from environment denied.");
+	    fprintf(stderr,"Color/mono text selection from environment denied.\n");
 	    break;
 	}
 	color_text = 0;
 	break;
     case 32:
 	if (!mode) {
-	    puts("Color/mono text selection from environment denied.");
+	    fprintf(stderr,"Color/mono text selection from environment denied.\n");
 	    break;
 	}
 	color_text = 1;
@@ -3069,7 +3085,7 @@ static char *process_option(int command, int mode, char **nptr)
 	    __svgalib_horizsync.max = f * 1000;
 	} else {
 	  hs_bad:
-	    printf("svgalib: Illegal HorizSync setting.\n"
+	    fprintf(stderr,"svgalib: Illegal HorizSync setting.\n"
 		   "Correct usage: HorizSync min_kHz max_kHz\n"
 		   "Example: HorizSync 31.5 36.5\n");
 	}
@@ -3092,7 +3108,7 @@ static char *process_option(int command, int mode, char **nptr)
 	    __svgalib_vertrefresh.max = f;
 	} else {
 	  vr_bad:
-	    printf("svgalib: Illegal VertRefresh setting.\n"
+	    fprintf(stderr,"svgalib: Illegal VertRefresh setting.\n"
 		   "Correct usage: VertRefresh min_Hz max_Hz\n"
 		   "Example: VertRefresh 50 70\n");
 	}
@@ -3178,7 +3194,7 @@ static char *process_option(int command, int mode, char **nptr)
 	}
     case 46:
 	if (!mode) {
-	    puts("Security setting from environment denied.");
+	    fprintf(stderr,"Security setting from environment denied.\n");
 	    break;
 	}
 	if ( (ptr = __svgalib_token(nptr)) ) {
@@ -3190,15 +3206,15 @@ static char *process_option(int command, int mode, char **nptr)
 		 break;
 	    }
 	} 
-	puts("svgalib: Unknown security options\n");
+	fprintf(stderr,"svgalib: Unknown security options\n");
 	break;
     case 47:
 	ptr = __svgalib_token(nptr);
 	if (ptr) {
-	    mouse_device = strdup(ptr);
-	    if (mouse_device == NULL) {
+	    __svgalib_mouse_device = strdup(ptr);
+	    if (__svgalib_mouse_device == NULL) {
 	      nomem:
-		puts("svgalib: Fatal error: out of memory.");
+		fprintf(stderr,"svgalib: Fatal error: out of memory.\n");
 		exit(1);
 	    }
 	} else
@@ -3210,12 +3226,12 @@ static char *process_option(int command, int mode, char **nptr)
 	  if (mode != -1) {
 	    __svgalib_default_mode = mode;
 	  } else {
-	    printf("svgalib: config: illegal mode \'%s\' for \'%s\'\n",
+	    fprintf(stderr,"svgalib: config: illegal mode \'%s\' for \'%s\'\n",
 	   			  ptr, vga_conf_commands[command]);
 	  }
 	} else {
   param_needed:
-  	  printf("svgalib: config: \'%s\' requires parameter(s)",
+  	  fprintf(stderr,"svgalib: config: \'%s\' requires parameter(s)",
   	  				vga_conf_commands[command]);
 	  break;
 	}
@@ -3239,7 +3255,7 @@ static char *process_option(int command, int mode, char **nptr)
 	if (!__joystick_devicenames[command - 51])
 	    goto nomem;
 #else
-	printf("svgalib: No joystick support in a.out version.\n");
+	fprintf(stderr,"svgalib: No joystick support in a.out version.\n");
 #endif
 	break;
     case 55: /* TextProg */
@@ -3267,7 +3283,7 @@ static char *process_option(int command, int mode, char **nptr)
         __svgalib_vesatext=1;
         break;
 #else
-       printf("svgalib: Warning: VESA support not enabled!\n");
+       fprintf(stderr,"svgalib: Warning: VESA support not enabled!\n");
 #endif
     case 57: /* Vesa save bitmap */  
 #ifdef INCLUDE_VESA_DRIVER
@@ -3277,7 +3293,7 @@ static char *process_option(int command, int mode, char **nptr)
          __svgalib_VESA_savebitmap=j;
        };
 #else
-       printf("svgalib: Warning: VESA support not enabled!\n");
+       fprintf(stderr,"svgalib: Warning: VESA support not enabled!\n");
 #endif
        break;
     case 58:
@@ -3291,7 +3307,7 @@ static char *process_option(int command, int mode, char **nptr)
 	        __svgalib_bandwidth = f;
 	    }
 	} else {
-  	    printf("svgalib: config: \'%s\' requires parameter(s)",
+  	    fprintf(stderr,"svgalib: config: \'%s\' requires parameter(s)",
   	  				vga_conf_commands[command]);
 	    break;
 	}
@@ -3322,7 +3338,7 @@ static char *process_option(int command, int mode, char **nptr)
         };
         break;
     case 62:
-        printf("svgalib: Warning: /proc/bus/pci is not used anymore, so option \"noprocpci\" ignored\n");
+        fprintf(stderr,"svgalib: Warning: /proc/bus/pci is not used anymore, so option \"noprocpci\" ignored\n");
         break;
     case 63: /* Vesa text mode number */  
 #ifdef INCLUDE_VESA_DRIVER
@@ -3332,7 +3348,7 @@ static char *process_option(int command, int mode, char **nptr)
          __svgalib_VESA_textmode=j;
        };
 #else
-       printf("svgalib: Warning: VESA support not enabled!\n");
+       fprintf(stderr,"svgalib: Warning: VESA support not enabled!\n");
 #endif
        break;
     case 64:			/* pci initial values */
@@ -3349,7 +3365,7 @@ static char *process_option(int command, int mode, char **nptr)
 	    if((j<256)&&(j>=0))__svgalib_pci_idev = j;
 	} else {
         ps_bad:
-	    printf("svgalib: Illegal PCI initial values setting.\n"
+	    fprintf(stderr,"svgalib: Illegal PCI initial values setting.\n"
 		   "Correct usage: PCIStart initial_bus initial_dev"
 		   "Example: PCIStart 1 0\n");
 	}
@@ -3362,7 +3378,7 @@ static char *process_option(int command, int mode, char **nptr)
 	    ptr = __svgalib_token(nptr);
             if ( (file = fopen(ptr, "r")) != 0) {
 #ifdef DEBUG_CONF
-                printf("Processing config file \'%s\'\n", ptr);
+                fprintf(stderr,"Processing config file \'%s\'\n", ptr);
 #endif
                 process_config_file(file, 1, vga_conf_commands, process_option);
                 fclose(file);
@@ -3379,11 +3395,11 @@ static char *process_option(int command, int mode, char **nptr)
 	if (ptr) {
 	    mem_device = strdup(ptr);
 	    if (mem_device == NULL) {
-		puts("svgalib: Fatal error: out of memory.");
+		fputs("svgalib: Fatal error: out of memory.\n",stderr);
 		exit(1);
 	    }
 	} else {
-  	  printf("svgalib: config: \'%s\' requires parameter(s)",
+  	  fprintf(stdout,"svgalib: config: \'%s\' requires parameter(s)",
   	  				vga_conf_commands[command]);
         }
 	break;
@@ -3404,19 +3420,19 @@ static void readconfigfile(void)
     __svgalib_read_options(vga_conf_commands, process_option);
     if (mouse_type == -1) {
 	mouse_type = MOUSE_MICROSOFT;	/* Default. */
-	puts("svgalib: Assuming Microsoft mouse.");
+	fprintf(stderr,"svgalib: Assuming Microsoft mouse.\n");
     }
     if (__svgalib_horizsync.max == 0U) {
 	/* Default monitor is low end SVGA/8514. */
 	__svgalib_horizsync.min = 31500U;
 	__svgalib_horizsync.max = 35500U;
-	puts("svgalib: Assuming low end SVGA/8514 monitor (35.5 KHz).");
+	fprintf(stderr,"svgalib: Assuming low end SVGA/8514 monitor (35.5 KHz).\n");
     }
 #ifdef DEBUG_CONF
-    printf("Mouse is: %d Monitor is: H(%5.1f, %5.1f) V(%u,%u)\n", mouse_type,
+    fprintf(stderr,"Mouse is: %d Monitor is: H(%5.1f, %5.1f) V(%u,%u)\n", mouse_type,
       __svgalib_horizsync.min / 1000.0, __svgalib_horizsync.max / 1000.0,
 	   __svgalib_vertrefresh.min, __svgalib_vertrefresh.max);
-    printf("Mouse device is: %s",mouse_device);
+    fprintf(stderr,"Mouse device is: %s",__svgalib_mouse_device);
 #endif
 }
 
@@ -3541,7 +3557,7 @@ int vga_init(void)
     if(__svgalib_secondary) __svgalib_emulatepage=1;
 
     if(__svgalib_driver_report) {
-        printf("svgalib %s\n", versionstr);
+        fprintf(stderr,"svgalib %s\n", versionstr);
     }
 
 #ifdef SET_TERMIO
@@ -3618,11 +3634,12 @@ int vga_init(void)
     };
    
 #ifdef DEBUG
-    printf("svgalib: Opening mouse (type = %x).\n", mouse_type | mouse_modem_ctl);
+    fprintf(stderr,"svgalib: Opening mouse (type = %x).\n", mouse_type | mouse_modem_ctl);
 #endif
     if(mouse_type != MOUSE_NONE ) {
-        if (mouse_init(mouse_device, mouse_type | mouse_modem_ctl, MOUSE_DEFAULTSAMPLERATE))
-           printf("svgalib: Failed to initialize mouse.\n");
+        __svgalib_mouse_flag=mouse_type | mouse_modem_ctl;
+        if (mouse_init(__svgalib_mouse_device, mouse_type | mouse_modem_ctl, MOUSE_DEFAULTSAMPLERATE))
+           fprintf(stderr,"svgalib: Failed to initialize mouse.\n");
         else
            mouse_open = 1;
     }
@@ -3690,7 +3707,7 @@ int vga_getcrtcregs(unsigned char *regs) {
 int vga_setcrtcregs(unsigned char *regs) {
     int i;
     
-    if(CM>9) return -1; 
+    if(!STDVGAMODE(CM)) return -1; 
     
     __svgalib_outcrtc(0x11,__svgalib_incrtc(0x11)&0x7f);
     for (i = 0; i < CRT_C; i++) {
