@@ -17,6 +17,8 @@ extern "C"
 {
 #endif
 
+#define SVGALIB_VER 0x010900
+
 #define TEXT 	     0		/* Compatible with VGAlib v1.2 */
 #define G320x200x16  1
 #define G640x200x16  2
@@ -193,9 +195,15 @@ extern "C"
 #define G400x600x16M 143
 #define G400x600x16M32 144
 
-#define __GLASTMODE G400x600x16M32
+#define G400x300x256X 145
+
+#define __GLASTMODE G400x300x256X
 #define GLASTMODE vga_lastmodenumber()
 
+#define IS_IN_STANDARD_VGA_DRIVER(mode) ( \
+	((mode) < G640x480x256) || ((mode) == G720x348x2) || \
+	( ((mode) >= G400x300x256X) && ((mode) <= G400x300x256X) ) )
+
     extern int vga_version;
 
     extern int vga_setmode(int mode);
@@ -278,6 +286,14 @@ extern "C"
 	/* depends from actual driver/chiptype.. etc. */
     } vga_modeinfo;
 
+    typedef struct {
+        int version;
+        int size;
+        int chipset;
+        int physmem;
+    } vga_cardinfo;
+
+    extern vga_cardinfo *vga_getcardinfo(void);
     extern vga_modeinfo *vga_getmodeinfo(int mode);
     extern int vga_getdefaultmode(void);
     extern int vga_getcurrentmode(void);
@@ -300,6 +316,7 @@ extern "C"
     extern void vga_disabledriverreport(void);
     extern int vga_setmodeX(void);
     extern int vga_init(void);	/* Used to return void in svgalib <= 1.12. */
+    extern int vga_initf(int);	
     extern int vga_getmousetype(void);
     extern int vga_getmonitortype(void);
     extern void vga_setmousesupport(int s);
@@ -316,6 +333,9 @@ extern "C"
     extern int vga_setlinearaddressing(void);
     extern void vga_safety_fork(void (*shutdown_routine) (void));
 
+    extern int vga_simple_init(void);
+    extern void vga_chipset_saveregs(unsigned char *);
+    extern void vga_chipset_setregs(unsigned char *);
 
 #ifdef EGA			/* Kernel headers may define this. */
 #undef EGA
@@ -348,6 +368,16 @@ extern "C"
 #define SIS		24
 #define I740		25
 #define NEOMAGIC	26
+#define LAGUNA		27
+#define FBDEV		28
+#define G400		29
+#define R128		30
+#define SAVAGE		31
+#define MILLENNIUM	32
+#define I810		33
+#define TRIDENT		34
+#define RENDITION 	35
+#define G450C2		36
 
     /* Hor. sync: */
 #define MON640_60	0	/* 31.5 KHz (standard VGA) */
@@ -377,6 +407,15 @@ extern "C"
     extern int vga_ext_set(unsigned what,...);
     extern int vga_accel(unsigned operation,...);
 
+    extern int vga_initcursor(int);
+    extern void vga_showcursor(int);
+    extern void vga_setcursorposition(int, int);
+    extern void vga_selectcursor(int);
+    extern void vga_setcursorimage(int, int, int, int, unsigned char *);
+
+    extern int vga_setcrtcregs(unsigned char *);
+    extern int vga_getcrtcregs(unsigned char *);
+
     extern int vga_addtiming(int pixelClock,
        			      int HDisplay,		
                               int HSyncStart,
@@ -415,6 +454,8 @@ extern "C"
 
    extern int vga_guesstiming(int x, int y, int clue, int arg);
 
+   extern void vga_dpms(int mode);
+
 /* Valid values for what in vga_ext_set: */
 #define VGA_EXT_AVAILABLE	0	/* supported flags */
 #define VGA_EXT_SET		1	/* set flag(s) */
