@@ -5,20 +5,25 @@
 #include "driver.h"
 
 static unsigned char *cursors[16];
+static int formats[16];
+
 static struct {
     unsigned char c8;
     unsigned short c15;
     unsigned short c16;
     unsigned int c32;
 } cursor_colors[16*2];
+
 static unsigned char *buf, *dbuf;
 static int cur_x, cur_y;
 static int cursor;
 static int cur_show;
-int __svgalib_software_cursor, __svgalib_cursor_status=0;
+int __svgalib_software_cursor, __svgalib_cursor_status=-1;
 static int palette[768];
 static int pal=1;
 
+static int sx, sy, sp;
+
 static int findcolor(int rgb) {
    int i,j,k,l=0;
 
@@ -47,7 +52,6 @@ static int software_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
     
     switch(cmd) {
         case CURSOR_INIT:
-            for (i=0;i<16;i++)cursors[i]=NULL;
             buf=(unsigned char *)malloc(32*32*4);
             dbuf=(unsigned char *)malloc(32*32*4);
             cursor=0;
@@ -62,7 +66,6 @@ static int software_cursor( int cmd, int p1, int p2, int p3, int p4, void *p5) {
             break;
         case CURSOR_SHOW:
             if(cur_show==0) {
-            /* maybe this should be allowed */
                 w=CI.bytesperpixel*32;
                 pattern=(unsigned int *)cursors[cursor];
                 for(i=0;i<32;i++)vga_getscansegment(buf+w*i,cur_x,cur_y+i,w);
@@ -194,21 +197,45 @@ void vga_showcursor(int show) {
 }
 
 void vga_setcursorposition(int x, int y) {
+    sx=x;
+    sy=y;
     chipset_cursor(CURSOR_POSITION,x,y,0,0,NULL);
 }
 
 void vga_setcursorimage(int cur, int format, int c0, int c1, unsigned char *buf) {
+    if(chipset_cursor!=software_cursor) {
+        /* save cursor for restoring on VT switch */
+        software_cursor(CURSOR_IMAGE,cur,format,c0,c1,buf);
+        formats[cur]=format;
+    }
     chipset_cursor(CURSOR_IMAGE,cur,format,c0,c1,buf);
 }
 
 void vga_selectcursor(int cur) {
+    sp=cur;
     chipset_cursor(CURSOR_SELECT,cur,0,0,0,NULL);
 }
 
 int vga_initcursor(int sw) {
-
+    int i;
     cur_show=0;
+    for (i=0;i<16;i++)cursors[i]=NULL;
+    __svgalib_cursor_status=0;
+
     if(sw || (chipset_cursor==NULL)) chipset_cursor=software_cursor;
-    return chipset_cursor(CURSOR_INIT,0,0,0,0,NULL);
+    i=chipset_cursor(CURSOR_INIT,0,0,0,0,NULL);
+    if(!i) chipset_cursor=software_cursor;
+    return chipset_cursor!=software_cursor;
+}
+
+void __svgalib_cursor_restore() {
+    int i;
+    
+    vga_selectcursor(sp);
+    vga_setcursorposition(sx, sy);
+    for (i=0;i<16;i++)if(cursors[i]!=NULL) {
+        chipset_cursor(CURSOR_IMAGE,i,formats[i],cursor_colors[i*2].c32,cursor_colors[i*2+1].c32,cursors[i]);
+    }
+    vga_showcursor(__svgalib_cursor_status&1);
 }
 
