@@ -129,7 +129,7 @@ static int verify_one_linear_mapping(int base, int size)
     }
     /* Linear framebuffer is OK. */
     if(__svgalib_driver_report)
-    printf("svgalib: Found linear framebuffer at 0x%08X.\n", base);
+    fprintf(stderr,"svgalib: Found linear framebuffer at 0x%08X.\n", base);
     result = 0;
    munmap((caddr_t) fb, size);
     (*lfn) (LINEAR_DISABLE, base);
@@ -202,7 +202,7 @@ int vga_setlinearaddressing(void)
 	mapaddr = (__svgalib_physmem() + (gran + gran - 1)) & ~(gran - 1);
 	maxmap = (range - 1) * gran;
 	if (mapaddr > maxmap) {
-	    puts("svgalib: Too much physical memory, cannot map aperture\n");
+	    fprintf(stderr,"svgalib: Too much physical memory, cannot map aperture\n");
 	    return -1;
 	}
 	if ((mappedMemory = verify_linear_mapping(mapaddr, memory)) == -1)
@@ -222,7 +222,7 @@ int vga_setlinearaddressing(void)
         graph_mem = LINEAR_POINTER;
 
 	if (memory != mappedMemory)
-	    printf("svgalib: Warning, card has %dK, only %dK available in linear mode.\n",
+	    fprintf(stderr,"svgalib: Warning, card has %dK, only %dK available in linear mode.\n",
 		   memory >> 10, mappedMemory >> 10);
 	return mappedMemory;
     }
